@@ -3,7 +3,10 @@
 
 import appdirs
 import argcomplete, argparse
+from cryptography.fernet import Fernet
+import getpass
 import json
+import keyring
 import os
 import pickle
 import re
@@ -28,19 +31,49 @@ def store_ladok_session(ls):
 
   file_path = dirs.user_cache_dir + "/LadokSession"
 
+  pickled_ls = pickle.dumps(ls)
+  key = keyring.get_password("ladok3", "cache_key")
+  if not key:
+    key = Fernet.generate_key()
+  keyring.set_password("ladok3", "cache_key", key)
+
+  fernet_protocol = Fernet(key)
+  encrypted_ls = fernet_protocol.encrypt(pickled_ls)
+
   with open(file_path, "wb") as file:
-    pickle.dump(ls, file)
+    file.write(encrypted_ls)
   
 def restore_ladok_session():
   file_path = dirs.user_cache_dir + "/LadokSession"
 
   if os.path.isfile(file_path):
     with open(file_path, "rb") as file:
-      return pickle.load(file)
+      encrypted_ls = file.read()
+      key = keyring.get_password("ladok3", "cache_key")
+
+      fernet_protocol = Fernet(key)
+      pickled_ls = fernet_protocol.decrypt(encrypted_ls)
+      return pickle.loads(pickled_ls)
 
   return None
-def load_credentials(filename):
+def update_credentials_in_keyring(ls, args):
+  user = input("LADOK username: ")
+  passwd = getpass.getpass("LADOK password: ")
+
+  keyring.set_password("ladok3", "username", user)
+  keyring.set_password("ladok3", "password", passwd)
+
+  clear_cache(ls, args)
+def load_credentials(filename="config.json"):
   """Load credentials from environment or file named filename"""
+  try:
+    user = keyring.get_password("ladok3", "username")
+    passwd = keyring.get_password("ladok3", "password")
+    if user and passwd:
+      return user, passwd
+  except:
+    pass
+
   try:
     user = os.environ["LADOK_USER"]
     passwd = os.environ["LADOK_PASS"]
@@ -53,14 +86,20 @@ def load_credentials(filename):
       config = json.load(conf_file)
     return config["username"], config["password"]
   except:
-    raise Exception("Cannot find credentials.")
+    pass
+
+  return None, None
 def clear_cache(ls, args):
-  os.remove(dirs.user_cache_dir + "/LadokSession")
+  try:
+    os.remove(dirs.user_cache_dir + "/LadokSession")
+  except FileNotFoundError as err:
+    pass
+
   sys.exit(0)
 def main():
   """Run the command-line interface for the ladok command"""
   argp = argparse.ArgumentParser(
-    description="Command-line interface for LADOK3",
+    description="This is a CLI-ification of LADOK3's web GUI.",
     epilog="Web: https://github.com/dbosk/ladok3"
   )
   argp.add_argument("-f", "--config-file",
@@ -73,6 +112,30 @@ def main():
     dest="command",
     required=True
   )
+  login_parser = subp.add_parser("login",
+    help="Manage login credentials",
+    description="""
+  Manages the user's LADOK login credentials. There are three ways to supply the 
+  login credentials, in order of priority:
+
+  1) Through the system keyring: Just run `ladok login` and you'll be asked to 
+     enter the credentials and they will be stored in the keyring.
+
+  2) Through the environment: Just set the environment variables LADOK_USER and 
+     LADOK_PASS.
+
+  3) Through the configuration file: Just write
+
+        {
+          "username": "the actual username",
+          "password": "the actual password"
+        }
+
+     to the file """ + dirs.user_config_dir + """/config.json (default, or use 
+     the -f option).
+  """
+  )
+  login_parser.set_defaults(func=update_credentials_in_keyring)
   cache_parser = subp.add_parser("cache",
     help="Manage cache",
     description="Manages the cache of LADOK data"
@@ -91,9 +154,9 @@ def main():
   ladok3.report.add_command_options(subp)
   argcomplete.autocomplete(argp)
   args = argp.parse_args()
-  LADOK_USER, LADOK_PASS = load_credentials(args.config_file)
   ls = restore_ladok_session()
   if not ls:
+    LADOK_USER, LADOK_PASS = load_credentials(args.config_file)
     ls = ladok3.LadokSessionKTH(LADOK_USER, LADOK_PASS)
   if "func" in args:
     args.func(ls, args)
